﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;

namespace howto_many_circle_arcs
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private List<PointF> Points = new List<PointF>();

        private void picCanvas_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
            e.Graphics.Clear(picCanvas.BackColor);

            // Find the circles.
            if (Points.Count == 2)
            {
                // Find and draw the circles.
                const int num_circles = 20;
                foreach (RectangleF rect in
                        FindCircles(Points[0], Points[1], num_circles))
                    e.Graphics.DrawEllipse(Pens.Red, rect);
            }

            // Draw the points.
            foreach (PointF point in Points)
                DrawPoint(e.Graphics, Brushes.Green, point);
        }

        private void picCanvas_MouseClick(object sender, MouseEventArgs e)
        {
            if (Points.Count == 2) Points = new List<PointF>();
            Points.Add(e.Location);
            picCanvas.Refresh();
        }

        private void DrawPoint(Graphics gr, Brush brush, PointF point)
        {
            const float radius = 3;
            RectangleF rect = new RectangleF(
                point.X - radius,
                point.Y - radius,
                2 * radius, 2 * radius);
            gr.FillEllipse(brush, rect);
        }

        // Find circles centered on a line perpendicular to the segment
        // between the two points and that intersect those points.
        private List<RectangleF> FindCircles(PointF point1, PointF point2, int num_circles)
        {
            // Get a scaled vector between the two points.
            float dx = point2.X - point1.X;
            float dy = point2.Y - point1.Y;
            float length = Distance(point1, point2);
            const float scale = 10;
            dx *= scale / length;
            dy *= scale / length;

            // Get the perpendicular vector.
            float p1x = -dy;
            float p1y = dx;
            float p2x = dy;
            float p2y = -dx;

            // Find the center point.
            float cx = (point1.X + point2.X) / 2f;
            float cy = (point1.Y + point2.Y) / 2f;

            // Generate circles.
            List<RectangleF> rects = new List<RectangleF>();
            for (int i = 0; i < num_circles; i++)
            {
                float new_cx = cx + i * p1x;
                float new_cy = cy + i * p1y;
                float radius = Distance(new PointF(new_cx, new_cy), point1);
                rects.Add(new RectangleF(
                    new_cx - radius, new_cy - radius,
                    2 * radius, 2 * radius));
            }
            return rects;
        }

        // Find the arcs connecting two points
        private void FindMidpointArcs(PointF point0, PointF point1,
            out RectangleF rect1, out RectangleF rect2,
            out float start_angle1, out float start_angle2,
            out float sweep_angle1, out float sweep_angle2)
        {
            // Find the midpoint.
            float cx = (point0.X + point1.X) / 2f;
            float cy = (point0.Y + point1.Y) / 2f;

            // Find the radius.
            float radius = (float)(Distance(point0, point1) / 2.0);

            // Create the rectangles.
            rect1 = new RectangleF(
                cx - radius, cy - radius,
                2 * radius, 2 * radius);
            rect2 = rect1;

            // Find the start angles.
            start_angle1 = (float)(180 / Math.PI * Math.Atan2(
                point1.Y - point0.Y,
                point1.X - point0.X));
            start_angle2 = (start_angle1 + 180) % 360;
            sweep_angle1 = 180;
            sweep_angle2 = 180;
        }

        private float Distance(PointF point1, PointF point2)
        {
            float dx = point1.X - point2.X;
            float dy = point1.Y - point2.Y;
            return (float)Math.Sqrt(dx * dx + dy * dy);
        }
    }
}
